@@ -22,6 +22,9 @@ from yellowdog_client.model import (
     FlattenPath,
     RunSpecification,
     Task,
+    TaskData,
+    TaskErrorMatcher,
+    TaskErrorType,
     TaskGroup,
     TaskInput,
     TaskInputSource,
@@ -53,17 +56,22 @@ from yellowdog_cli.utils.printing import (
     print_log,
     print_numbered_strings,
     print_warning,
+    print_yd_object,
 )
 from yellowdog_cli.utils.property_names import *
 from yellowdog_cli.utils.settings import (
     MAX_BATCH_SUBMIT_ATTEMPTS,
+    MAX_PARALLEL_TASK_BATCH_UPLOAD_THREADS,
     NAMESPACE_OBJECT_STORE_PREFIX_SEPARATOR,
     VAR_CLOSING_DELIMITER,
+    VAR_NAME_OF_UNNAMED_TASK,
     VAR_OPENING_DELIMITER,
 )
 from yellowdog_cli.utils.submit_utils import (
     UploadedFiles,
+    generate_task_error_matchers_list,
     generate_task_input_list,
+    generate_taskdata_object,
     pause_between_batches,
     update_config_work_requirement_object,
 )
@@ -520,6 +528,9 @@ def create_task_group(
                 NAMESPACES, wr_data.get(NAMESPACES, config_wr.namespaces)
             )
         ),
+        retryableErrors=generate_task_error_matchers_list(
+            config_wr, wr_data, task_group_data
+        ),
     )
     ctttl_data = check_float_or_int(
         task_group_data.get(
@@ -627,8 +638,20 @@ def add_tasks_to_task_group(
 
     num_submitted_tasks = 0
 
+    # Determine number of parallel upload threads
+    parallel_upload_threads = (
+        CONFIG_WR.parallel_batches
+        if ARGS_PARSER.parallel_batches is None
+        else ARGS_PARSER.parallel_batches
+    )
+    parallel_upload_threads = (
+        MAX_PARALLEL_TASK_BATCH_UPLOAD_THREADS
+        if parallel_upload_threads is None
+        else parallel_upload_threads
+    )
+
     # Single batch or sequential batch submission
-    if ARGS_PARSER.parallel_batches == 1 or num_task_batches == 1:
+    if parallel_upload_threads == 1 or num_task_batches == 1:
         if num_task_batches > 1:
             print_log(f"Uploading {num_task_batches} Task batches sequentially")
         for batch_number in range(num_task_batches):
@@ -667,7 +690,7 @@ def add_tasks_to_task_group(
             print_warning(
                 "Option 'pause-between-batches/-P' is ignored for parallel batch uploads"
             )
-        max_workers = min(num_task_batches, ARGS_PARSER.parallel_batches)
+        max_workers = min(num_task_batches, parallel_upload_threads)
         print_log(
             f"Submitting Task batches using {max_workers} parallel submission threads"
         )
@@ -736,18 +759,33 @@ def generate_batch_of_tasks_for_task_group(
     for task_number in range(start_task_number, end_task_number):
         task_group_data = wr_data[TASK_GROUPS][tg_number]
         task = tasks[task_number] if task_count is None else tasks[0]
-        task_name = format_yd_name(
-            get_task_name(
-                task.get(NAME, task.get(TASK_NAME, CONFIG_WR.task_name)),
-                task_number,
-                num_tasks,
-                tg_number,
-                num_task_groups,
-                task_group.name,
+
+        set_task_names = check_bool(
+            task.get(
+                SET_TASK_NAMES,
+                task_group_data.get(
+                    SET_TASK_NAMES,
+                    wr_data.get(SET_TASK_NAMES, CONFIG_WR.set_task_names),
+                ),
             )
         )
 
-        add_or_update_substitution(L_TASK_NAME, str(task_name))
+        task_name = get_task_name(
+            task.get(NAME, task.get(TASK_NAME, CONFIG_WR.task_name)),
+            set_task_names,
+            task_number,
+            num_tasks,
+            tg_number,
+            num_task_groups,
+            task_group.name,
+        )
+
+        task_name = None if task_name is None else format_yd_name(task_name)
+
+        add_or_update_substitution(
+            L_TASK_NAME,
+            VAR_NAME_OF_UNNAMED_TASK if task_name is None else task_name,
+        )
         add_or_update_substitution(
             L_TASK_NUMBER, formatted_number_str(task_number, num_tasks)
         )
@@ -971,10 +1009,33 @@ def generate_batch_of_tasks_for_task_group(
             ),
         )
 
-        # Set 'alwaysUpload' for all outputs
+        # Set 'alwaysUpload' for all object store outputs
         for task_output in outputs:
             task_output.alwaysUpload = always_upload
 
+        # Data client inputs and outputs
+        task_data_inputs = check_list(
+            task.get(
+                TASK_DATA_INPUTS,
+                task_group_data.get(
+                    TASK_DATA_INPUTS,
+                    wr_data.get(TASK_DATA_INPUTS, config_wr.task_data_inputs),
+                ),
+            )
+        )
+        task_data_outputs = check_list(
+            task.get(
+                TASK_DATA_OUTPUTS,
+                task_group_data.get(
+                    TASK_DATA_OUTPUTS,
+                    wr_data.get(TASK_DATA_OUTPUTS, config_wr.task_data_outputs),
+                ),
+            )
+        )
+        task_data_inputs_and_outputs = generate_taskdata_object(
+            task_data_inputs, task_data_outputs
+        )
+
         # If there's no task type in the task definition, AND
         # there's only one task type at the task group level,
         # use that task type
@@ -1014,6 +1075,7 @@ def generate_batch_of_tasks_for_task_group(
                 flatten_upload_paths=flatten_upload_paths,
                 flatten_input_paths=flatten_input_paths,
                 add_yd_env_vars=add_yd_env_vars,
+                task_data_inputs_and_outputs=task_data_inputs_and_outputs,
             )
         )
 
@@ -1136,6 +1198,8 @@ def get_task_data_property(
             ) as f:
                 return f.read()
 
+    return None
+
 
 def follow_progress_old(work_requirement: WorkRequirement) -> None:
     """
@@ -1180,13 +1244,13 @@ def on_update(work_req: WorkRequirement):
 
 def cleanup_on_failure(work_requirement: WorkRequirement) -> None:
     """
-    Clean up the Work Requirement and any uploaded Objects on failure
+    Clean up the Work Requirement and any uploaded Objects on failure.
     """
     if ARGS_PARSER.dry_run:
         return
 
     CLIENT.work_client.cancel_work_requirement(work_requirement)
-    print_log(f"Cancelled Work Requirement '{work_requirement.name}'")
+    print_warning(f"Cancelled Work Requirement '{work_requirement.name}'")
 
     # Delete uploaded objects
     UPLOADED_FILES.delete()
@@ -1195,7 +1259,7 @@ def cleanup_on_failure(work_requirement: WorkRequirement) -> None:
 def deduplicate_inputs(task_inputs: List[TaskInput]) -> List[TaskInput]:
     """
     Deduplicate a list of TaskInputs. This is useful when wildcards
-    are used. Note that TaskInputs that differ only in their verification
+    are used. Note that TaskInputs which differ only in their verification
     type will be caught by 'check_for_duplicates_in_file_lists()'.
     """
     deduplicated_task_inputs: List[TaskInput] = []
@@ -1245,12 +1309,13 @@ def formatted_number_str(
 
 def get_task_name(
     name: Optional[str],
+    set_task_names: bool,
     task_number: int,
     num_tasks: int,
     task_group_number: int,
     num_task_groups: int,
     task_group_name: str,
-) -> str:
+) -> Optional[str]:
     """
     Create the name of a Task.
     Supports lazy substitution.
@@ -1278,9 +1343,12 @@ def get_task_name(
             task_group_name,
         )
 
-    else:
+    elif set_task_names:
         name = "task_" + formatted_number_str(task_number, num_tasks)
 
+    else:
+        name = None
+
     return name
 
 
@@ -1320,7 +1388,7 @@ def create_task(
     wr_data: Dict,
     task_group_data: Dict,
     task_data: Dict,
-    task_name: str,
+    task_name: Optional[str],
     task_number: int,
     tg_name: str,
     tg_number: int,
@@ -1335,6 +1403,7 @@ def create_task(
     flatten_upload_paths: bool = False,
     flatten_input_paths: Optional[FlattenPath] = None,
     add_yd_env_vars: bool = False,
+    task_data_inputs_and_outputs: Optional[TaskData] = None,
 ) -> Task:
     """
     Create a Task object, handling special processing for specific Task Types.
@@ -1350,17 +1419,21 @@ def create_task(
         if inputs is None or len(inputs) == 0:
             flatten_input_paths = None
 
+        if task_name is None and len(outputs) > 0:
+            raise Exception(f"Tasks must be named if outputs are specified")
+
         return Task(
             name=task_name,
             taskType=task_type,
-            arguments=args,
-            inputs=inputs,
-            environment=env_copy,
-            outputs=outputs,
+            arguments=None if len(args) == 0 else args,
+            inputs=None if len(inputs) == 0 else inputs,
+            environment=None if len(env_copy) == 0 else env_copy,
+            outputs=None if len(outputs) == 0 else outputs,
             flattenInputPaths=flatten_input_paths,
             taskData=task_data_property,
             timeout=task_timeout,
             tag=task_tag,
+            data=task_data_inputs_and_outputs,
         )
 
     task_tag = task_data.get(TASK_TAG, None)
@@ -1676,9 +1749,7 @@ def submit_json_task_batch(
             f"{CONFIG_COMMON.namespace}/requirements/{wr_name}/"
             f"taskGroups/{task_group_name}/tasks"
         ),
-        headers={
-            "Authorization": (f"yd-key {CONFIG_COMMON.key}:{CONFIG_COMMON.secret}")
-        },
+        headers={"Authorization": f"yd-key {CONFIG_COMMON.key}:{CONFIG_COMMON.secret}"},
         json=task_batch,
     )
 
