@@ -494,13 +494,6 @@ class CLIParser:
                     " level prefixes"
                 ),
             )
-            parser.add_argument(
-                "--details",
-                "-d",
-                action="store_true",
-                required=False,
-                help="show the full details of (interactively) selected objects",
-            )
             parser.add_argument(
                 "--work-requirements",
                 "-w",
@@ -547,14 +540,14 @@ class CLIParser:
                 ),
             )
             parser.add_argument(
-                "--compute-templates",
+                "--compute-requirement-templates",
                 "-C",
                 action="store_true",
                 required=False,
                 help="list compute requirement templates",
             )
             parser.add_argument(
-                "--source-templates",
+                "--compute-source-templates",
                 "-S",
                 action="store_true",
                 required=False,
@@ -651,16 +644,35 @@ class CLIParser:
                 required=False,
                 help="list roles",
             )
+            parser.add_argument(
+                "--permissions",
+                action="store_true",
+                required=False,
+                help="list permissions",
+            )
+            parser.add_argument(
+                "--details",
+                "-d",
+                action="store_true",
+                required=False,
+                help="show the full JSON representation of objects",
+            )
             parser.add_argument(
                 "--output-file",
                 type=str,
                 required=False,
                 help=(
-                    "if specified, the 'details' resource listing will be appended "
+                    "if specified, the 'details' JSON resource listing will also be written "
                     "to the nominated output file"
                 ),
                 metavar="<output-file>",
             )
+            parser.add_argument(
+                "--auto-select-all",
+                action="store_true",
+                required=False,
+                help="use with '--details' to select all objects automatically",
+            )
 
         if any(module in sys.argv[0] for module in ["upload"]):
             parser.add_argument(
@@ -1159,8 +1171,17 @@ class CLIParser:
                 action="store_true",
                 required=False,
                 help=(
-                    "substitute compute source template IDs and image family IDs for "
-                    "names in detailed compute requirement templates"
+                    "use with '--details' to substitute compute source template IDs and "
+                    "image family IDs for names in detailed compute requirement templates"
+                ),
+            )
+            parser.add_argument(
+                "--strip-ids",
+                action="store_true",
+                required=False,
+                help=(
+                    "use with '--details' to omit the YellowDog IDs of objects from "
+                    "their JSON representations"
                 ),
             )
 
@@ -1180,6 +1201,13 @@ class CLIParser:
                 nargs="+",
                 help="the YellowDog ID(s) of the worker pool(s) to compare",
             )
+            parser.add_argument(
+                "--running-nodes-only",
+                "-R",
+                action="store_true",
+                required=False,
+                help="only include RUNNING nodes in the specified worker pools in the comparison",
+            )
 
         self.args = parser.parse_args()
 
@@ -1321,6 +1349,10 @@ class CLIParser:
     def details(self) -> Optional[bool]:
         return self.args.details
 
+    @details.setter
+    def details(self, interactive: bool):
+        self.args.details = interactive
+
     @property
     @allow_missing_attribute
     def debug(self) -> Optional[bool]:
@@ -1478,13 +1510,13 @@ class CLIParser:
 
     @property
     @allow_missing_attribute
-    def compute_templates(self) -> Optional[bool]:
-        return self.args.compute_templates
+    def compute_requirement_templates(self) -> Optional[bool]:
+        return self.args.compute_requirement_templates
 
     @property
     @allow_missing_attribute
-    def source_templates(self) -> Optional[bool]:
-        return self.args.source_templates
+    def compute_source_templates(self) -> Optional[bool]:
+        return self.args.compute_source_templates
 
     @property
     @allow_missing_attribute
@@ -1551,6 +1583,11 @@ class CLIParser:
     def roles(self) -> Optional[bool]:
         return self.args.roles
 
+    @property
+    @allow_missing_attribute
+    def permissions(self) -> Optional[bool]:
+        return self.args.permissions
+
     @property
     @allow_missing_attribute
     def show_keyring_passwords(self) -> Optional[bool]:
@@ -1711,6 +1748,21 @@ class CLIParser:
     def terminate(self) -> Optional[bool]:
         return self.args.terminate
 
+    @property
+    @allow_missing_attribute
+    def running_nodes_only(self) -> Optional[bool]:
+        return self.args.running_nodes_only
+
+    @property
+    @allow_missing_attribute
+    def strip_ids(self) -> Optional[bool]:
+        return self.args.strip_ids
+
+    @property
+    @allow_missing_attribute
+    def auto_select_all(self) -> Optional[bool]:
+        return self.args.auto_select_all
+
 
 def lookup_module_description(module_name: str) -> Optional[str]:
     """
