@@ -41,6 +41,7 @@ from yellowdog_client.model import (
     Node,
     ObjectDetail,
     ObjectPath,
+    PermissionDetail,
     ProvisionedWorkerPool,
     ProvisionedWorkerPoolProperties,
     Role,
@@ -70,6 +71,8 @@ from yellowdog_cli.utils.settings import (
     MAX_LINES_COLOURED_FORMATTING,
     MAX_TABLE_DESCRIPTION,
     NAMESPACE_OBJECT_STORE_PREFIX_SEPARATOR,
+    PROP_CREATED_TIME,
+    PROP_ID,
     PROP_RESOURCE,
     WARNING_STYLE,
 )
@@ -230,6 +233,7 @@ TYPE_MAP = {
     NamespacePolicy: "Namespace Policy",
     Node: "Node",
     ObjectPath: "Object Path",
+    PermissionDetail: "Permission",
     ProvisionedWorkerPool: "Provisioned Worker Pool",
     Task: "Task",
     TaskGroup: "Task Group",
@@ -831,6 +835,29 @@ def roles_table(
     return headers, table
 
 
+def permissions_table(
+    permissions: List[PermissionDetail],
+) -> (List[str], List[List]):
+    headers = [
+        "#",
+        "Name",
+        "Description",
+        "Includes",
+    ]
+    table = []
+    for index, permission in enumerate(permissions):
+        includes = ", ".join(sorted([x for x in permission.includes]))
+        table.append(
+            [
+                index + 1,
+                permission.name,
+                permission.title,
+                includes,
+            ]
+        )
+    return headers, table
+
+
 def print_numbered_object_list(
     client: PlatformClient,
     objects: List[Union[Item, str, Dict]],
@@ -899,6 +926,8 @@ def print_numbered_object_list(
         headers, table = groups_table(objects)
     elif isinstance(objects[0], Role):
         headers, table = roles_table(objects)
+    elif isinstance(objects[0], PermissionDetail):
+        headers, table = permissions_table(objects)
     else:
         table = []
         for index, obj in enumerate(objects):
@@ -970,6 +999,9 @@ def sorted_objects(
         except TypeError:
             return objects
 
+    if isinstance(objects[0], Task):  # Sort tasks by their task number
+        return sorted(objects, key=lambda x: int(x.id.split(":")[-1]), reverse=reverse)
+
     try:
         return sorted(objects, key=lambda x: x.name, reverse=reverse)
     except:
@@ -1032,6 +1064,30 @@ def print_yd_object(
     using the compact JSON encoder.
     """
     object_data: object = Json.dump(yd_object)
+
+    def remove_unused_props(d):
+        """
+        Helper function to remove the 'id' and 'createdTime' properties
+        recursively.
+        """
+        if isinstance(d, dict):
+            # Create a new dictionary omitting the 'id' and 'createdTime'
+            # keys
+            return {
+                k: remove_unused_props(v)
+                for k, v in d.items()
+                if k not in [PROP_ID, PROP_CREATED_TIME]
+            }
+        elif isinstance(d, list):
+            # Recursively process each item in the list
+            return [remove_unused_props(item) for item in d]
+        else:
+            # Return non-dict/list values unchanged
+            return d
+
+    if ARGS_PARSER.strip_ids:
+        object_data = remove_unused_props(object_data)
+
     if add_fields is not None:
         # Requires a copy of the 'object' datatype to be made,
         # in order to insert additional fields
@@ -1041,6 +1097,7 @@ def print_yd_object(
         for key, value in add_fields.items():
             object_data_new[key] = value
         object_data = object_data_new
+
     print_json(object_data, initial_indent, drop_first_line, with_final_comma)
 
 
